#!/bin/bash
# (C) Copyright 2007-2013 Hewlett-Packard Development Company, L.P.
# Version: A.12.00.00
#
# This script contains Serviceguard NFS toolkit defined environment
# variables and functions that are used for following:
# 1. Starting and Stopping NFS daemons.
# 2. Exporting and Unexporting NFS shares.
# 3. Starting and Stopping rmtab synchronization process.
# 4. Starting and Stopping File lock migration process.
#
# Implementation:
# 1) Modification was made to support the file lock migration feature.
# 2) Modification was made to support the version 4 of the NFS protocol 
#    for NFS server.( 09/02/2012 ) 
# 3) Modification was made to merge the lock migration script functionalities
#    as a part of startup based on lock migration attributes. ( 18/03/2012 )
# 4) Modification was made to merge the sync_rmtab binary functionalities 
#    into hanfs.sh script in the form of shell script functions. sync_rmtab
#    binary file has been removed. ( 28/08/2012 )
# 5) Added fix for the quix QXCR1001265600 and QXCR1001265598 ( 10/04/2013 )
#
# Exit value definition:
# 1 - a failure occurs in one of the function.
#     
#
#Include the SG PATHs

. /etc/cmcluster.conf

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/bin:

# Define number of NFS listener daemons to be launched
RPCNFSDCOUNT=8

# The NFS_FLM_SCRIPT variable is the name of the script that manages
# synchronizing the file lock status entries for the primary and
# adoptive nodes associated with this NFS Package.
NFS_FLM_SCRIPT="$TKIT_SCRIPT_DIR/nfs.flm"

# START OF RUN FUNCTIONS

# Definitions for script internal use. Do not modify this part.
NFS_RMTAB_FILE=/var/lib/nfs/rmtab
SYNC_RMTAB_PID_FILE="$TKIT_SCRIPT_DIR/sync_rmtab.PID.$SG_PACKAGE"
NFS_EXTAB_LOCK=/var/lock/extab.lock



#
# Call the script responsible for populating the sm (/var/lib/nfs/sm for SLES
# and /var/lib/nfs/statd/sm for RH  ) and holding directories with Status
# Monitor entries. For NFSv4, this script is responsible for populating the
# /var/lib/nfs/v4recovery and holding directories. 
#
# Make sure the FLM script exists and is executable before launching it. Also
# kill any currently running copy of the script before launching a new copy.
#
# Pass in the command-line option of either "populate_sm_dir" to instruct the
# FLM script to copy the SM entries from the holding directory to the
# sm directory, or "populate_holding_dir" to continuously populate the
# holding directory with SM entries from sm directory.
# If we are populating the sm directory then wait for the script to complete
# and return.  If we are populating the holding directory then launch the
# script in the background as the script will not return until it is killed.
#

function call_FLM_script {

if [[ -f $NFS_FLM_SCRIPT ]]
then
        if [[ ! -x $NFS_FLM_SCRIPT ]]
        then
                chmod 544 $NFS_FLM_SCRIPT
                if [[ $? -ne 0 ]]
                then
                        echo "$(date '+%b %e %T') - Node \"$(hostname)\": Failed to change permissions of \"$NFS_FLM_SCRIPT\".  Exiting."
                        #
                        # `let 0` is used to set the value of $? to 1.
                        # The function check_return requires $? to
                        # be set to 1 if it has to print an error message.
                        #
                        let 0
                        check_return 32
                fi
        fi
else
        echo "$(date '+%b %e %T') - Node \"$(hostname)\": NFS File LockMigration Script \"${NFS_FLM_SCRIPT##*/}\" does not exist, exiting."
        #
        # `let 0` is used to set the value of $? to 1.
        # The function check_return requires $? to
        # be set to 1 if it has to print an error message.
        #
        let 0
        check_return 32
fi

kill_FLM_script ${NFS_FLM_SCRIPT##*/}

echo "$(date '+%b %e %T') - Node \"$(hostname)\": Calling NFS FLM Script \"${NFS_FLM_SCRIPT##*/}\" with parameter \"$1\""

if [[ $1 = "populate_sm_dir" ]]
then
        ${NFS_FLM_SCRIPT} $1
        check_return 32

elif [[ $1 = "populate_holding_dir" ]]
then
        ${NFS_FLM_SCRIPT} $1 &
        check_return 32
fi
}

function kill_FLM_script
{
        for k in /proc/*
        do
                if [ ! -f $k/stat ]; then
                        continue
                fi
        pid=`grep "($1)" $k/stat`
                if [ ! -z "$pid" ]; then
                       echo -e "\tKilling the FLM Script ${k##*/}"
                       kill -9 ${k##*/}
                fi
        done
}


# This function starts the NFS daemons
#
function nfs_defined_run_cmds
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting NFS daemons."

    # start each NFS daemon if it was not running
    if [[ -z ${NFSv4_VIRTUAL_ROOT} ]] ; then
    if [[ $release -eq 0 ]] || [[ ! -z "$IS_SUSE" && $suse_version -eq 11 ]]; then
      pidlist=`pidofproc rpcbind`
      if [[ -z $pidlist ]]; then
          echo -e "\tStarting rpcbind daemon"
          /sbin/rpcbind 
          check_return 16
      else
          echo -e "\trpcbind had been running on this node."
      fi
    else 
      pidlist=`pidofproc portmap`
      if [[ -z $pidlist ]]; then
          echo -e "\tStarting portmap daemon"
          portmap
          check_return 16
      else
          echo -e "\tportmap had been running on this node."
      fi
     fi

    # Checking for Suse or Red hat Linux
    if [[ -z "$IS_SUSE" ]] || [[ $suse_version -eq 11 ]] ;
    then
        #This is a Red Hat or Suse 11 machine run rpc.statd if required
        pidlist=`pidofproc rpc.statd`
        if [[ -z $pidlist ]]; then     # execute if not already running
            echo -e "\tStarting rpc.statd daemon"
            rpc.statd
            check_return 16
        else
            echo -e "\trpc.statd had been running on this node."
        fi

    else
        #This is a SuSE machine run sm-notity if required
        pidlist=`pidofproc sm-notify`
        if [[ -z $pidlist ]];         # execute if not already running
        then
            echo -e "\tStarting sm-notify daemon"
            sm-notify
            check_return 16
        else
           echo -e "\tsm-notify had been running on this node."
        fi
    fi

    if [ $QUOTA_MON == "YES" ]; then
        pidlist=`pidofproc rpc.rquotad`

        if [[ -z $pidlist ]]; then
            echo -e "\tStarting rpc.rquotad daemon"
            rpc.rquotad
            check_return 16
        else
            echo -e "\trpc.rquotad had been running on this node."
        fi
    fi
  fi

    # Only following daemons are needed for NFSv4. If package is 
    # configured for NFSv4, then following daemons will be started.
    # If package is configured for NFSv3/NFSv2, along with above daemons,
    # below daemons will also be started. 

    #rpcbind is needed for SLES 11 as it supports ipv6.
   if [[ ! -z "${IS_SUSE}" ]] && [[ $suse_version -eq 11 ]] && [[ ! -z ${NFSv4_VIRTUAL_ROOT} ]]; then
      pidlist=`pidofproc rpcbind`
      if [[ -z $pidlist ]]; then
          echo -e "\tStarting rpcbind daemon"
          /sbin/rpcbind
          check_return 16
      else
          echo -e "\trpcbind had been running on this node."
      fi
   fi 

 
    pidlist=`pidofproc nfsd`
    if [[ -z $pidlist ]]; then
        echo -e "\tStarting $RPCNFSDCOUNT nfsd daemons"
        rpc.nfsd $RPCNFSDCOUNT
        check_return 16
    else
        echo -e "\tnfsd had been running on this node."
    fi

    pidlist=`pidofproc rpc.mountd`
    if [[ -z $pidlist ]]; then
        echo -e "\tStarting rpc.mountd daemon"
        rpc.mountd
        check_return 16
        sleep 1
    else
        echo -e "\trpc.mountd had been running on this node."
    fi

    if [[ -n ${NFSv4_VIRTUAL_ROOT} ]]; then
       pidlist=`pidofproc rpc.idmapd`
       if [[ -z $pidlist ]]; then
         echo -e "\tStarting rpc.idmapd daemon"
         rpc.idmapd 
         check_return 16
       else
         echo -e "\trpc.idmapd had been running on this node"
       fi
    fi
      
}


# This function kills NFS daemons.
#
function nfs_defined_halt_cmds
{
# it is not necessary to stop NFS daemons. They may be started on boot time
# or currently used by other packages.
if [ $LOCK_MIGRATION == "YES" ]; then
    if [[ -z ${NFSv4_VIRTUAL_ROOT} ]]; then 
      #Release all the locks to unmount the filesystem successfully
      if [[ $release -eq 1 ]]; then 
       killproc lockd
      fi

      if [[ ${QUOTA_MON} == "YES" ]]; then
        echo -e "\tStoping rpc.rquotad daemon"
       killproc rpc.rquotad
      fi

      # for Redhat machine or Suse 11 machine restart statd
      if [[ $suse_version -eq 11 ]] || [[ -z "$IS_SUSE" ]]; then
         echo -e "\tRestarting statd daemon"
         killproc rpc.statd
         rpc.statd
      else
         echo -e "\tKilling sm-notify"
         killproc sm-notify
      fi
    
    else
       echo -e "\tStoping idmapd daemon"
       killproc rpc.idmapd
    fi
   
    echo -e "\tStoping rpc.mountd daemon"
    killproc rpc.mountd

    echo -e "\tStoping nfsd daemon"
    killproc nfsd

    kill_FLM_script  ${NFS_FLM_SCRIPT##*/}
fi
}


# This function checks if the XFS is a logic volume mounted file system.
#
function check_lv_fs
{

    #Function modified for using awk instead of grep for more specific search
    # of mount VGs.
    # Grep will give confict with vgNFS1 & vgNFS11

    mountinfo="$1"
    result=`mount | awk '$3 == filesys {print $1}' filesys="$mountinfo"`

    echo $result
}


# For each exported file system, export it with the options
# using exportfs -i(1m).
#
function export_fs
{
    for I in "${XFS[@]}"
    do
        while :
        do
            echo "$(date '+%b %e %T') - Node \"$(hostname)\": Exporting filesystem on ${I##*:}"
            ISMOUNTED=`check_lv_fs ${I##*:}`
            if [[ -z $ISMOUNTED ]]; then
                echo -e "\tWARNING: ${I##*:} is not mounted on a volume group"
            fi

            # Lock the exportfs table to prevent multiple packages to export/unexport
            # file systems at the same time.
            lockfile -l 15 $NFS_EXTAB_LOCK
            EXPORTERROR=`exportfs -i $I 2>&1`
            # unlock the exportfs table
            rm -f $NFS_EXTAB_LOCK

            XTAB_IN_USE=`echo $EXPORTERROR | grep "xtab in use"`
            if [[ -n "$XTAB_IN_USE" ]]; then
                echo -e "\tWARNING: exportfs -i $I: $EXPORTERROR. Retrying in 2 seconds"
                sleep 2
            else
                if [[ -n "$EXPORTERROR" ]]; then
                    echo -e "\t$EXPORTERROR"
                    # unlock the exportfs table
                    rm -f $NFS_EXTAB_LOCK
                    [[ 1 == 0 ]]
                    check_return 17
                fi
                break
            fi
        done
    done
}

# This function exports only NFSv4 supported export directories.
# Export and unexport mechanism is different in NFSv4, hence  
# exporting/unexporting is not clubbed with NFSv3/NFSv2 type
# of exporting/unexporting.

function export_fs_nfsv4
{

#Create NFSV4 directory if it does not exist.
if [[ ! -d ${NFSv4_VIRTUAL_ROOT} ]]; then
  mkdir -p $NFSv4_VIRTUAL_ROOT
  chmod 755 $NFSv4_VIRTUAL_ROOT
fi

#Create other export directories and bind them to directories that needs to be mounted 
#at NFS client.
for I in "${XFS[@]}"
do
   #we only want directory names. not export options.
    I=${I##*:}
    mkdir -p ${NFSv4_VIRTUAL_ROOT}${I}
    mount --bind $I ${NFSv4_VIRTUAL_ROOT}${I}
done

#Exporting the virtual root directory. For virtual root directory, fsid should be 0.
#Having Multiple virtual roots is not supported as it causes unexpected behavior by NFS.
exportfs -i -o fsid=0 *:${NFSv4_VIRTUAL_ROOT}
if [[ $? -ne 0 ]]; then
  echo -e "\tError: Failed to export NFSv4 virtual root directory ${NFSv4_VIRTUAL_ROOT}"
fi

for I in "${XFS[@]}"
    do
        while :
        do
            echo "$(date '+%b %e %T') - Node \"$(hostname)\": Exporting filesystem on ${I##*:}"
            ISMOUNTED=`check_lv_fs ${I##*:}`
            if [[ -z $ISMOUNTED ]]; then
                echo -e "\tWARNING: ${I##*:} is not mounted on a volume group"
            fi

            # Lock the exportfs table to prevent multiple packages to export/unexport
            # file systems at the same time.
            # prefix_string extracts the export options of the export directories.
            prefix_string=`echo "$I" | cut -d "/" -f 1`
            I=${I##*:}

            lockfile -l 15 $NFS_EXTAB_LOCK
            EXPORTERROR=`exportfs -i ${prefix_string}${NFSv4_VIRTUAL_ROOT}${I} 2>&1`
            # unlock the exportfs table
            rm -f $NFS_EXTAB_LOCK
            XTAB_IN_USE=`echo $EXPORTERROR | grep "xtab in use"`
            if [[ -n "$XTAB_IN_USE" ]]; then
                echo -e "\tWARNING: exportfs -i ${NFSv4_VIRTUAL_ROOT}${I}: $EXPORTERROR. Retrying in 2 seconds"
                sleep 2
            else
                if [[ -n "$EXPORTERROR" ]]; then
                    echo -e "\t$EXPORTERROR"
                    # unlock the exportfs table
                     rm -f $NFS_EXTAB_LOCK
                    [[ 1 == 0 ]]
                    check_return 17
                fi
                break
            fi
        done
    done
}
 
                    

# Unexport file system
#
function un_export_fs
{
    for I in "${XFS[@]}"
    do
        # This was added because in the case of unexport
        # we only want the dir name
        #
        I=${I##* }
        while :
        do
            echo "$(date '+%b %e %T') - Node \"$(hostname)\": Unexporting filesystem on $I"

            # Lock the exportfs table to prevent multiple packages to export/unexport
            # file systems at the same time.
            lockfile -l 15 $NFS_EXTAB_LOCK
            EXPORTERROR=`exportfs -u $I 2>&1`
            # unlock the exportfs table
            rm -f $NFS_EXTAB_LOCK

            XTAB_IN_USE=`echo $EXPORTERROR | grep "xtab in use"`
            if [[ -n "$XTAB_IN_USE" ]]; then
                echo -e "\tWARNING: exportfs -u $I: $EXPORTERROR. Retrying in 2 seconds"
                sleep 2
            else
                if [[ -n "$EXPORTERROR" ]]; then
                    echo "$EXPORTERROR"
                    [[ 1 == 0 ]]
                    check_return 18
                fi
                break
            fi
        done
    done
}

# This function unexports only NFSv4 exported directories.

function un_export_fs_nfsv4
{
   for I in "${XFS[@]}"
    do
        # This was added because in the case of unexport
        # we only want the dir name
        #
        I=${I##* }
        while :
        do
            echo "$(date '+%b %e %T') - Node \"$(hostname)\": Unexporting filesystem on $I"

            # Lock the exportfs table to prevent multiple packages to export/unexport
            # file systems at the same time.

            #get prefix string.
            prefix_string=`echo "$I" | cut -d "/" -f 1`
            I=${I##*:}

            lockfile -l 15 $NFS_EXTAB_LOCK
            EXPORTERROR=`exportfs -u ${prefix_string}${NFSv4_VIRTUAL_ROOT}${I} 2>&1`
            # unlock the exportfs table
            rm -f $NFS_EXTAB_LOCK
            umount ${NFSv4_VIRTUAL_ROOT}${I}
            if [[ $? -eq 0 ]]; then
               rm -rf ${NFSv4_VIRTUAL_ROOT}${I}
            else
               echo -e "\tWARNING: Failed to unmount ${NFSv4_VIRTUAL_ROOT}${I}" 
            fi

            XTAB_IN_USE=`echo $EXPORTERROR | grep "xtab in use"`
            if [[ -n "$XTAB_IN_USE" ]]; then
                echo -e "\tWARNING: exportfs -u ${prefix_string}${NFSv4_VIRTUAL_ROOT}${I}: $EXPORTERROR. Retrying in 2 seconds"
                sleep 2
            else
              if [[ -n "$EXPORTERROR" ]]; then
                    echo "$EXPORTERROR"
                    [[ 1 == 0 ]]
                    check_return 18
                fi
                break
            fi
        done
    done

   # Check if this virtual root is used by other packages in the system. If yes
   # do not unexport it.
   /bin/mount -v | /bin/grep "${NFSv4_VIRTUAL_ROOT}" >> /dev/null
   if [[ $? -ne 0 ]] ; then
     exportfs -u *:${NFSv4_VIRTUAL_ROOT}
   fi

}

# Find all pid of a process
#
function pidofproc
{
    pidof $1
}


# kill the named process
#
function killproc
{
    pidlist=`pidofproc $1`
    if [[ -n $pidlist ]]; then
        for pid in "$pidlist"; do
            kill -15 $pid
        done
    fi    
    sleep 1
    pidlist=`pidofproc $1`     
    if [[ -n $pidlist ]]; then 
       for pid in "$pidlist"; do  
          kill -9 $pid
          check_return 21
       done 
    fi    
}

# Check return value of functions and exit.
#
function check_return
{
    if (( $? != 0 ))
    then
        case $1 in
            16)
                echo "ERROR: Function nfs_defined_run_cmds"
                echo "ERROR: Failed to start a NFS daemon"
                nfs_defined_halt_cmds
                if [[ -z ${NFSv4_VIRTUAL_ROOT} ]]; then
                  un_export_fs
                  synchronize_rmtab export
                else
                  un_export_fs_nfsv4
                fi
                exit 1
                ;;

            17)
                if [[ -z ${NFSv4_VIRTUAL_ROOT} ]]; then
                  echo "ERROR: Function export_fs"
                  echo "ERROR: Failed to export $I"
                  un_export_fs
                  synchronize_rmtab export
                else
                   echo "ERROR: Function export_fs_nfsv4"
                   echo "ERROR: Failed to export $I" 
                   un_export_fs_nfsv4
                fi 
                exit 1
                ;;

            18)
                if [[ -z ${NFSv4_VIRTUAL_ROOT} ]]; then 
                  echo "ERROR: Function un_export_fs"
                else
                  echo "ERROR: Function un_export_fs_nfsv4"
                fi
                echo "ERROR: Failed to unexport $I"
                exit_value=1 
                ;;

            21)
                echo "ERROR: Function nfs_defined_halt_cmds"
                echo "ERROR: Failed to stop nfs daemon"
                exit_value=1
                ;;

            31)
                echo "ERROR: Function stop_sync_rmtab_bg_proc"
                echo "ERROR: Failed to stop sync rmtab background process"
                exit_value=1
                ;;
            32)
                echo "ERROR:  Function call_FLM_script"
                echo "ERROR:  Failed to call_FLM_script $I"
                if [[ -z ${NFSv4_VIRTUAL_ROOT} ]] ; then
                  stop_sync_rmtab_bg_proc
                  nfs_defined_halt_cmds
                  un_export_fs
                  synchronize_rmtab export
                else
                  nfs_defined_halt_cmds
                  un_export_fs_nfsv4
                fi
                exit 1
                ;;
            33)
                echo "ERROR:Improper parameters in package configuration file."
                exit 1
                ;;
            *)
                echo "ERROR: Failed, unknown error."
                ;;
        esac
    fi
}

# This function will be used by the rmtab synchronization
# process for locking rmtab file, sync file and temp file
# while performing the operations like import, export and
# sync of the files.

function lock_file
{
if [[ -x /usr/bin/lockfile ]];then
  /usr/bin/lockfile -l 15 $1.lock
  if [[ $? -ne 0 ]]; then
     echo "ERROR: lock_file: Failed to lock the file $1"
     exit 1
  fi
else
  echo "ERROR: lock_file: lockfile command does not exist"
  exit 1
fi
}

# This Function will be used by the rmtab synchronization
# process for unlocking rmtab file, sync file and temp file
# after operations like write and read are complete.

function unlock_file
{

if [[ -f $1.lock ]]; then
  rm -f $1.lock
fi

}

# This function will copy the contents from the sync file in the
# shared storage to the /var/lib/nfs/rmtab file. This function uses
# directory mentioned in the XFS[0] attribute value to store the sync file.

function import_rmtab
{

if [[ ! -f $SYNC_FILE ]]; then
  echo "import_rmtab: $SYNC_FILE does not exist"
  return 0
fi

lock_file $RMTAB_PATH
rsync $SYNC_FILE $RMTAB_PATH 
if [[ $? -ne 0 ]]; then
  echo "WARNING: import_rmtab: Failed to copy the contents from sync file in shared storage to /var/lib/nfs/rmtab file"
  unlock_file $RMTAB_PATH
  return 1
fi
unlock_file $RMTAB_PATH
}

# This function will sync the contents from the /var/lib/nfs/rmtab to
# the sync file in the shared storage.

function sync_rmtab_file
{

if [[ ! -f $SYNC_FILE ]]; then
  echo "sync_rmtab_file: $SYNC_FILE does not exist"
fi

lock_file $RMTAB_PATH
lock_file $SYNC_FILE
if [[ -x /usr/bin/rsync ]]; then
  rsync $RMTAB_PATH $SYNC_FILE
  if [[ $? -ne 0 ]]; then
    echo "ERROR: sync_rmtab_file: Failed to sync the contents from rmtab file to sync_file"
    unlock_file $RMTAB_PATH
    unlock_file $SYNC_FILE
    return 1
  fi
fi
unlock_file $RMTAB_PATH
unlock_file $SYNC_FILE

}


# This function will alter the contents of /var/lib/nfs/rmtab file.
# Since we have to modify the /var/lib/nfs/rmtab file contents,
# its safer to use temporary file instead of directly operating
# on /var/lib/nfs/rmtab. Hence temporary file will be used.

function alter_rmtab
{

lock_file $RMTAB_PATH
lock_file $TEMP_FILE

rsync $RMTAB_PATH $TEMP_FILE
if [[ $? -ne 0 ]]; then
  echo "ERROR: alter_rmtab: Copying contents from /var/lib/nfs/rmtab file to temp file failed"
  return 1
fi
rsync $TEMP_FILE $RMTAB_PATH
if [[ $? -ne 0 ]]; then
  echo "ERROR: alter_rmtab: Copying the contents from the temp file to /var/lib/nfs/rmtab file failed"
  return 1
fi
unlock_file $RMTAB_PATH
unlock_file $TEMP_FILE

}

# This function will copy the client mount entries from /var/lib/nfs/rmtab to
# sync file which is in shared storage. This function also removes the entries from the
# /var/lib/nfs/rmtab.

function export_rmtab
{

sync_rmtab_file
if [[ $? -ne 0 ]]
then
  echo "ERROR: export_rmtab: syncing rmtab file failed"
  return 1
fi

alter_rmtab
if [[ $? -ne 0 ]]
then
 echo "ERROR: export_rmtab: altering rmtab file failed"
 return 1
fi

}

# This function will be called by synchronize_rmtab function. 

function sync_rmtab_function {

case $1 in
    import)
          import_rmtab
          return $?
          ;;

    sync)
           sync_rmtab_file
           return $? 
           ;;

    export)

           export_rmtab
           return $?
           ;;
    *)
           echo "sync_rmtab_function: Usage: sync_rmtab import|sync|export exported-file-system"
           return 1
           ;;
esac

}

# This function is provided for the rmtab file synchronization purpose.
# It can be called for exporting clients current mounted information to a cluster
# shared storage. It can be called for importing most recent client mounted information
# from the shared storage to the local rmtab file. It also can be called for refreshing
# the sync file which stored in the shared storage.
# The main goal of this function is to achieve clients seamless failover on
# NFS file system mount and unmount.
#
# This function takes one parameter (export, import, or sync)
# Note: NFSv4 does not populates rmtab file with client entries. So
# This function is not required to be called for NFSv4 server packages.
#
function synchronize_rmtab
{
    local temp=${XFS[0]}
    local dest=${temp##*:}


    # Check if an exported shared storage exist
    if [[ -z $dest ]]
    then
        echo "ERROR: sync rmtab: No exported file system on this package."
        return 1
    fi

   
    #Define the paths of the files used by rmtab synchronization process
    dest=`df -P $dest | tail -1 | awk '{print $6}'` 
    RMTAB_PATH="/var/lib/nfs/rmtab"
    SYNC_FILE_PATH=$dest
    SYNC_FILE=${SYNC_FILE_PATH}/.rmtab_sync
    TEMP_FILE="/var/lib/nfs/rmtab_temp"
 

    # Check if the source/destination directory is a mounted volume group
    if [[ -z "$dest" ]]
    then
        echo "ERROR: sync rmtab: exported file system is not a mounted volume group."
        return 1
    fi


    sync_rmtab_function $1 
    return $?

}


# Periodically check if there is a new client mount point. Update
# the rmtab syncronization file if there is a change.
# This function will be called and run as a background task
# Note: NFSv4 does not populates rmtab file with client entries. So
# This function is not required to be called for NFSv4 server packages.

function sync_rmtab_periodically
{

        local timestamp_prev="";
        local timestamp_curr="";

        while :; do
                timestamp_curr=`ls -l --time-style=full-iso $NFS_RMTAB_FILE | awk '{print $6$7}'`;
                if [[ $timestamp_curr != $timestamp_prev ]]
                then
                        synchronize_rmtab sync
                        if (( $? != 0 ))
                        then
                                exit 1;
                        fi
                fi
                timestamp_prev=$timestamp_curr;
                sleep 2;
        done
}


# Start synchronizing rmtab background process
#
# Note: NFSv4 does not populates rmtab file with client entries. So
# This function is not required to be called for NFSv4 server packages.

function start_sync_rmtab_bg_proc
{
    # This function assume that each HA package maintains its own directory.
    # i.e. no package shares directory. Otherwise this method will
    # be messed up.
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting rmtab synchronization process"
    sync_rmtab_periodically &
    echo $! > $SYNC_RMTAB_PID_FILE
}


# Stop synchronizing rmtab background process
#
# Note: NFSv4 does not populates rmtab file with client entries. So
# This function is not required to be called for NFSv4 server packages.

function stop_sync_rmtab_bg_proc
{
    if [[ -f $SYNC_RMTAB_PID_FILE ]]; then
        syncrmtabpid=`cat $SYNC_RMTAB_PID_FILE`
        if [[ -n $syncrmtabpid ]]; then
            echo "$(date '+%b %e %T') - Node \"$(hostname)\": Stoping rmtab synchronization process"
            kill -9 $syncrmtabpid
            check_return 31
            rm -f $SYNC_RMTAB_PID_FILE
        else
            echo "WARNING: Stoping rmtab synchronization proces: $SYNC_RMTAB_PID_FILE file corrupted"
        fi
    else
        echo "WARNING: Stoping rmtab synchronization proces: $SYNC_RMTAB_PID_FILE does not exist"
    fi
}

###############################################################################
# Function check_yes_no
#
# This function echoes the String depending upon the condition variable value
# The condition variable should be either yes or no
# Parameter 1: Name of the Condition Variable
# Parameter 2: Condition variable value ( either yes/no)
# Parameter 3: String to be echoed
# Return 0 if condition 2 is either yes/no otherwise return 1
#
# If parameter 2 is "yes" then parameter 3 is echoed
# If parameter 2 is "no" then parameter 3 is not echoed
###############################################################################
function check_yes_no
{
    local arg;
    local arg1;

    if [ -z "$2" ]; then
        arg="invalid"
    else
        arg=`echo $2 | tr "[:lower:]" "[:upper:]" | tr -d "[:space:]"`
        fi

    if [ "$2" = "$3" ]; then
        arg1=$arg
    else
        arg1=$3
    fi

    case $arg in
        YES)
            echo $arg1
            ;;
        NO)
            [ "$arg1" = "$arg" ] && echo $arg1
            ;;
        *)
            echo "Invalid option $1=$2. $1 can be set either as $1=\"YES\" or $1
=\"NO\""
            return 1
            ;;
    esac
    return 0
}



#
#function to validate package attributes
#
function validate_parameters
{
        LOCK_MIGRATION=`check_yes_no LOCK_MIGRATION "$LOCK_MIGRATION" "$LOCK_MIGRATION"`
        check_status=$?
        if [ $check_status -ne 0 ]; then
             echo "$LOCK_MIGRATION"
             let 0
             check_return 33
        fi

        if [[ $LOCK_MIGRATION="yes"  && -n $NFS_FLM_MONITOR ]]; then
          NFS_FLM_MONITOR=`check_yes_no NFS_FLM_MONITOR "$NFS_FLM_MONITOR" "$NFS_FLM_MONITOR"`
          check_status=$?
          if [ $check_status -ne 0 ]; then
            echo "$NFS_FLM_MONITOR"
            let 0
            check_return 33
          fi
        fi

        QUOTA_MON=`check_yes_no QUOTA_MON "$QUOTA_MON" "$QUOTA_MON"`
        check_status=$?
        if [ $check_status -ne 0 ]; then
            echo "$QUOTA_MON"
            let 0
            check_return 33
        fi
}


#-------------------MAINLINE NFS Code Starts Here-----------------
#
# FUNCTION STARTUP SECTION.  HV

exit_value=0
typeset syncrmtabpid
typeset flm_pid
validate_parameters

#Check if redhat release 6 or greater.
if [[ -x /usr/bin/lsb_release ]]; then
  if [[ -f /etc/redhat-release ]]; then
      release_num=`/usr/bin/lsb_release -r | cut -d ":" -f 2`
      release_num=`echo $release_num | cut -d "." -f 1`
      if [[ $release_num -ge 6 ]] ; then
        release=0
      else
        release=1
      fi
  elif [[ -f /etc/SuSE-release ]];then
     IS_SUSE=`cat /proc/version | grep -i "SuSE" | grep -v "grep"`
     if [[ ! -z "${IS_SUSE}" ]]; then
       suse_version=`/usr/bin/lsb_release -r | cut -d ":" -f 2`
     fi
  else
     echo "ERROR: /etc/redhat-release or /etc/SuSE-release file specific to distro is missing"
     exit 1
  fi
else
  echo "ERROR: /usr/bin/lsb_release file does not exist"
  echo "ERROR: Unable to determine the Distro release version"
  exit 1
fi
Action=`echo $1 | awk -F":" '{print $1}'`
set $Action

# Test to see if we are being called to run the package, or halt the package.
if [[ $Action = "start" ]]
then
    
    if [[ -z ${NFSv4_VIRTUAL_ROOT} ]];then
       synchronize_rmtab import
       if (( $? != 0 ))
       then
         exit 1
       fi
       export_fs
       nfs_defined_run_cmds
       start_sync_rmtab_bg_proc
    else
       export_fs_nfsv4
       nfs_defined_run_cmds
    fi
    
    #Check if lock migration is enabled, if yes, then the following part will be executed.
    if [ $LOCK_MIGRATION == "YES" ];then
       # Determine the package floating IP address. If multiple floating IP's are configured,
       # then first floating IP will be used.
       PACKAGE_IP=`$SGSBIN/cmgetpkgenv $SG_PACKAGE | grep "SG_IP_ADDRESS\[0\]" | cut -d "\"" -f 2 2>/dev/null`
 
       if [[ ! -z $PACKAGE_IP ]]; then
          PkgIP="$PACKAGE_IP"
       else
          if [[ -x /usr/bin/host ]]; then
             NODE=`/usr/bin/host -T $NODE`
             if [[ $? -eq 0 ]]; then
               NODE_IP_ADDRESS=`echo "$NODE" | cut -d " " -f 4`
               PkgIP="$NODE_IP_ADDRESS"
             else
               echo "ERROR: /usr/bin/host command failed to determine the node IP address"
             fi
          else
             echo "ERROR: /usr/bin/host command not found"
             echo "ERROR: Unable to find the Node IP address"
          fi
       fi

       call_FLM_script populate_sm_dir
       if [[ -z ${NFSv4_VIRTUAL_ROOT} ]]; then
          echo -e "Shutting down NFS file locking daemon "
          killproc lockd
          if [[ -z "$IS_SUSE" ]] || [[ $suse_version -eq 11 ]];
          then
            echo -e "Restarting rpc.statd daemon using virtual IP $PkgIP"
            killproc rpc.statd
            rpc.statd -n $PkgIP
          else
            killproc sm-notify
            echo -e "Starting sm-notify using virtual IP $PkgIP"
            sm-notify -v $PkgIP
          fi
       fi

        call_FLM_script populate_holding_dir
    else
        echo "$(date '+%b %e %T') - Node \"$(hostname)\": The parameter LOCK_MIGRATION is not set. NFS Client Locks will not be migrated." 
        exit 0
    fi
      
elif [[ $Action = "stop" ]]
then

#rpcbind is needed for a clean shutdown in sles11
    if [[ ! -z ${IS_SUSE} ]] && [[ $suse_version -eq 11 ]]; then
      echo "Starting rpcbind to ensure the clear shutdown of other daemons" 
      service rpcbind start
    fi
    
    if [[ -z ${NFSv4_VIRTUAL_ROOT} ]];then
      stop_sync_rmtab_bg_proc
      un_export_fs
      nfs_defined_halt_cmds
      synchronize_rmtab export
      if (( $? != 0 ))
      then
        exit_value=1
      fi
    else
       un_export_fs_nfsv4
       nfs_defined_halt_cmds
    fi
    
fi

exit $exit_value
